package com.xypower.mpmaster.sms;

import android.Manifest;
import android.app.Activity;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.telephony.SmsManager;
import android.telephony.SmsMessage;
import android.telephony.SubscriptionInfo;
import android.telephony.SubscriptionManager;
import android.text.TextUtils;

import androidx.core.app.ActivityCompat;

import org.json.JSONArray;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.TimeZone;


/*
 * 短信接收类
 *
 * */
public class SimUtil {
    public static final String SMS_RECEIVED_ACTION = "android.provider.Telephony.SMS_RECEIVED";
    public static final String SMS_DELIVER_ACTION = "android.provider.Telephony.SMS_DELIVER";

    //自定义ACTION常数 作为广播的IntentFilter识别常数
    public static String SMS_SEND_ACTION = "com.xypower.mpmaster.SMS_SEND_ACTION";
    public static String SMSTYPE = "smstype";

    //短信解析
    public static void analysisSMSInfo(Intent intent, Context context) {
        SmsMessageModel smsInfo = getSMSInfo(intent);
        if (smsInfo != null) {
            String content = smsInfo.getContent();
            int slot = smsInfo.getSlot();
            String sender = smsInfo.getSender();
            String sendmessage = "";
            String sendtype = "";
            if (StringUtils.isEmpty(content)) {
                return;
            }
            if (content.contains(SmsTypeEnum.REBOOT1.value())) {
                sendmessage = content + " OK";
                sendtype = SmsTypeEnum.REBOOT1.value();
            } else if (content.contains(SmsTypeEnum.REBOOT2.value())) {
                sendmessage = content + " OK";
                sendtype = SmsTypeEnum.REBOOT2.value();
            } else if (content.contains(SmsTypeEnum.SET_YW_SCHEDULE.value())) {
                String packageName = context.getApplicationContext().getPackageName();
                String[] split1 = StringUtils.splitString1(content);
                if (split1 != null && split1.length > 1) {
                    String s = split1[1];
                    String[] split2 = StringUtils.splitString2(s);
                    if (split2 != null && split2.length >= 1) {
                        String num = split2[0];
                        Integer integer = StringUtils.convert2Int(num);
                        if (integer != null) {
                            if (integer == 0) {//删除所有运维
                                UpdateSysConfigUtil.updateAbsHeartbeats(packageName, new JSONArray());
                            } else {
                                JSONArray jsonArray = new JSONArray();
                                int times = 0;
                                for (int i = 0; i < split2.length; i++) {
                                    if (i == 0) {
                                        continue;
                                    }
                                    String ts = split2[i];
                                    Integer time = StringUtils.convert2Int(ts);
                                    if (i % 2 == 1) {
                                        time = 0;
                                        times = time * 3600;
                                    } else {
                                        times += time * 60;
                                        jsonArray.put(times);
                                    }
                                }
                            }
                        }
                    }
                }
                sendmessage = content + " OK";
                sendtype = SmsTypeEnum.REBOOT2.value();
            }
            sendSms(context, slot, sender, sendmessage, sendtype);
        }
    }

    //短信解析
    public static SmsMessageModel getSMSInfo(Intent intent) {
        SmsMessageModel smsMessageModel = new SmsMessageModel();
        String action = intent.getAction();
        if (SMS_RECEIVED_ACTION.equals(action) || SMS_DELIVER_ACTION.equals(action)) {
            Bundle bundle = intent.getExtras();
            if (bundle != null) {
                String content = "";
                String sender = "";
                String dateContent = "";
                Object[] pdus = (Object[]) bundle.get("pdus");
                if (pdus != null && pdus.length > 0) {
                    SmsMessage[] messages = new SmsMessage[pdus.length];
                    for (int i = 0; i < pdus.length; i++) {
                        byte[] pdu = (byte[]) pdus[i];
                        messages[i] = SmsMessage.createFromPdu(pdu);
                    }
                    for (SmsMessage message : messages) {
                        content = message.getMessageBody();// 得到短信内容
                        // 得到发信息的号码
                        sender = message.getOriginatingAddress();
                        Date date = new Date(message.getTimestampMillis());
                        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                        format.setTimeZone(TimeZone.getTimeZone("GMT+08:00"));
                        dateContent = format.format(date);
                    }
                    int smsSlot = getSmsSlot(bundle);
                    smsMessageModel.setContent(content);
                    smsMessageModel.setSender(sender);
                    smsMessageModel.setDateContent(dateContent);
                    smsMessageModel.setSlot(smsSlot);
                }
            }
        }
        return smsMessageModel;
    }

    //获取是哪个sim卡收到短信
    private static int getSmsSlot(Bundle bundle) {
        int slot = -1;
        Set<String> keySet = bundle.keySet();
        for (String key : keySet) {
            switch (key) {
                case "slot":
                    slot = bundle.getInt("slot", -1);
                    break;
                case "simId":
                    slot = bundle.getInt("simId", -1);
                    break;
                case "simSlot":
                    slot = bundle.getInt("simSlot", -1);
                    break;
                case "slot_id":
                    slot = bundle.getInt("slot_id", -1);
                    break;
                case "simnum":
                    slot = bundle.getInt("simnum", -1);
                    break;
                case "slotId":
                    slot = bundle.getInt("slotId", -1);
                    break;
                case "slotIdx":
                    slot = bundle.getInt("slotIdx", -1);
                    break;
                default:
                    if (key.toLowerCase().contains("slot") | key.toLowerCase().contains("sim")) {
                        String value = bundle.getString(key, "-1");
                        if (value.equals("0") | value.equals("1") | value.equals("2")) {
                            slot = bundle.getInt(key, -1);
                        }
                    }
            }
        }
        return slot;
    }

    //指定sim卡位置发送短信
    public static void sendSms(Context mContext, int slot, String sender, String message, String value) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP_MR1) {
            SubscriptionManager localSubscriptionManager = SubscriptionManager.from(mContext);
            if (ActivityCompat.checkSelfPermission(mContext, Manifest.permission.READ_PHONE_STATE) != PackageManager.PERMISSION_GRANTED) {
                return;
            }
            if (localSubscriptionManager.getActiveSubscriptionInfoCount() > 1) {
                List localList = localSubscriptionManager.getActiveSubscriptionInfoList();

                SubscriptionInfo simInfo1 = (SubscriptionInfo) localList.get(0);
                SubscriptionInfo simInfo2 = (SubscriptionInfo) localList.get(1);
                SubscriptionInfo simInfoAnother;
                if (slot == 0) {
                    simInfoAnother = simInfo1;
                } else {
                    simInfoAnother = simInfo2;
                }
                Intent itSend = new Intent(SMS_SEND_ACTION);
                itSend.putExtra(SMSTYPE, value);
                //sendIntent参数为传送后接受的广播信息PendingIntent
                PendingIntent sendPI = PendingIntent.getBroadcast(mContext, 0, itSend, 0);

//                Intent itDeliver = new Intent(SMS_DELIVERED_ACTION);
                //deliveryIntent参数为传送后接受的广播信息PendingIntent
//                PendingIntent deliverPI = PendingIntent.getBroadcast(mContext,0,itDeliver,0);
                SmsManager.getSmsManagerForSubscriptionId(simInfoAnother.getSubscriptionId()).sendTextMessage(sender, null, message, sendPI, null);
            }
        }
    }


    //判断是否是双卡
    public static boolean isDoubleSim(Context mContext) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP_MR1) {
            SubscriptionManager manager = SubscriptionManager.from(mContext);
            if (ActivityCompat.checkSelfPermission(mContext, Manifest.permission.READ_PHONE_STATE) != PackageManager.PERMISSION_GRANTED) {
                return false;
            }
            int count = manager.getActiveSubscriptionInfoCount();
            if (count == 2) {
                return true;
            }
        }
        return false;
    }

    //通过卡槽id获取sim卡的信息0代码卡槽1，1代表卡槽2
    public static String getSlotIdInfo(Context mContext, int slotId) {
        String info = "";
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP_MR1) {
            SubscriptionManager manager = SubscriptionManager.from(mContext);
            if (ActivityCompat.checkSelfPermission(mContext, Manifest.permission.READ_PHONE_STATE) != PackageManager.PERMISSION_GRANTED) {
                return "";
            }
            SubscriptionInfo subInfo = manager.getActiveSubscriptionInfoForSimSlotIndex(slotId);
            if (subInfo != null) {
                info = subInfo.getCarrierName().toString();
            }
        }
        return info;
    }

    //判断手机中是否装了双卡
    public static boolean isHasDoubleSim(Context mContext) {
        try {
            if (isDoubleSim(mContext)) {
                List<ModelSlotAndSub> slotAndSubs = getModelSlot(mContext);
                if (slotAndSubs != null && slotAndSubs.size() == 2) {
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }


    //判断是否拥有该权限
    public static boolean IsHasPermission(Context mContext, String permission) {
        if (mContext != null) {
            if (ActivityCompat.checkSelfPermission(mContext, permission) != PackageManager.PERMISSION_GRANTED) {
                return false;
            }
        } else {
            return false;
        }
        return true;
    }


    //申请该权限
    public static void requestOnePermission(Activity activity, String permission, int permissionCode) {
        if (activity != null) {
            activity.requestPermissions(new String[]{permission}, permissionCode);
        }
    }


    //获取icc用来判断
    public static int getIcc(String iccid, List<ModelSlotAndSub> slotAndSubs) {
        int icc = 0;
        if (slotAndSubs != null && slotAndSubs.size() >= 2 && !TextUtils.isEmpty(iccid)) {
            if (iccid.startsWith(slotAndSubs.get(0).iccid)) {
                icc = 1;
            } else if (iccid.startsWith(slotAndSubs.get(1).iccid)) {
                icc = 2;
            } else {
                if (iccid.equals(slotAndSubs.get(0).subId)) {
                    icc = 1;
                } else if (iccid.equals(slotAndSubs.get(1).subId)) {
                    icc = 2;
                } else {
                    icc = 0;
                }
            }
        }
        return icc;
    }

    /**
     * 获取来自那张卡 即卡1 还是卡2
     *
     * @param mContext
     * @param subID
     * @return
     */
    public static int getSlot(Context mContext, String subID) {
        return getIcc(subID, getModelSlot(mContext));
    }

    //获取对应的卡槽ID和iccID 关联
    private static List<ModelSlotAndSub> getModelSlot(Context mContext) {
        List<ModelSlotAndSub> data = new ArrayList<>();
        ModelSlotAndSub modelSlotAndSub = null;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP_MR1) {
            SubscriptionManager subscriptionManager = SubscriptionManager.from(mContext);
            if (ActivityCompat.checkSelfPermission(mContext, Manifest.permission.READ_PHONE_STATE) == PackageManager.PERMISSION_GRANTED) {
                List<SubscriptionInfo> mSubcriptionInfos = subscriptionManager.getActiveSubscriptionInfoList();
                if (mSubcriptionInfos != null) {
                    for (int i = 0; i < mSubcriptionInfos.size(); i++) {
                        SubscriptionInfo info = mSubcriptionInfos.get(i);
                        if (info != null) {
                            modelSlotAndSub = new ModelSlotAndSub();
                            modelSlotAndSub.subId = info.getSubscriptionId() + "";
                            modelSlotAndSub.simSlot = info.getSimSlotIndex() + 1 + "";
                            modelSlotAndSub.iccid = info.getIccId();
                            data.add(modelSlotAndSub);
                        }
                    }
                    if (data != null && data.size() == 2) {
                        if (!TextUtils.isEmpty(data.get(0).simSlot) && !TextUtils.isEmpty(data.get(1).simSlot)) {
                            int simSlot1 = Integer.valueOf(data.get(0).simSlot);
                            int simSlot2 = Integer.valueOf(data.get(1).simSlot);
                            if (simSlot1 > simSlot2) {
                                ModelSlotAndSub modelSlotAndSub1 = data.get(1);
                                data.remove(data.get(1));
                                data.add(0, modelSlotAndSub1);
                            }
                        }
                    }
                }
            }
        }
        return data;
    }

}